<?php class RigakuImport extends Device {

    const SCORING_SYSTEM_NAME='Rigaku';

    const RIGAKU_SCORES= array(
        array('color'=>'ffffff', 'index'=>'0', 'hotkey'=>'0', 'name'=>'Clear'),
        array('color'=>'aeaeae', 'index'=>'1', 'hotkey'=>'9', 'name'=>'Matter'),
        array('color'=>'c800ff', 'index'=>'2', 'hotkey'=>'8', 'name'=>'Phase separation'),
        array('color'=>'ff0041', 'index'=>'3', 'hotkey'=>'7', 'name'=>'Heavy precipitation'),
        array('color'=>'ff8200', 'index'=>'4', 'hotkey'=>'6', 'name'=>'Precipitation'),
        array('color'=>'009b00', 'index'=>'5', 'hotkey'=>'5', 'name'=>'Spherulites'),
        array('color'=>'00ff1b', 'index'=>'6', 'hotkey'=>'4', 'name'=>'Clusters'),
        array('color'=>'00ffeb', 'index'=>'7', 'hotkey'=>'3', 'name'=>'Crystalline'),
        array('color'=>'94c6ff', 'index'=>'8', 'hotkey'=>'2', 'name'=>'Micro-crystals'),
        array('color'=>'0064fa', 'index'=>'9', 'hotkey'=>'1', 'name'=>'Crystals')
    );

    /**
     * @param $params
     * @return array
     * @throws BadRequestException
     * @throws ForbiddenException
     * @throws NotFoundException
     * @throws ServerException
     */
    public static function createScores($params){
        Log::debug('In RigakuImport::createScores');
        foreach($params as $k=>$v){
            Log::debug('- '.$k.'='.$v);
        }
        if(!isset($params['name'])){
            Log::info('Setting default scoring system name '.static::SCORING_SYSTEM_NAME);
            $params['name']=static::SCORING_SYSTEM_NAME;
        }
        if(!isset($params['scores'])){
            Log::info('Setting default Rigaku scores');
            $params['scores']=static::RIGAKU_SCORES;
        }
        $scoringSystem=ImageScoring::createScoringSystem($params);
        Log::debug('Returning from RigakuImport::createScores');
        return $scoringSystem;
    }

    /**
     * @throws ServerException
     * @throws NotFoundException
     * @throws BadRequestException
     */
    public static function getScores(){
        Log::debug('In RigakuImport::getScores');
        $scores=ImageScoring::getScoresWithScoringSystemName(static::SCORING_SYSTEM_NAME);
        Log::debug('Returning from RigakuImport::getScores');
        return $scores;
    }

    /**
     * @throws BadRequestException
     * @throws ForbiddenException
     * @throws NotFoundException
     * @throws ServerException
     */
    public static function createUser($user){
        Log::debug('In RigakuImport::createUser');
        foreach($user as $k=>$v){
            Log::debug('- '.$k.'='.$v);
        }
        static::require('username',$user);
        $username=$user['username'];
        $usernameSanitized=preg_replace('/[^A-Za-z0-9_.-]/','',$username);
        if(!$username!==$usernameSanitized){
            Log::warning('Username contains illegal characters. Sanitized.');
            Log::warning("Supplied: $username");
            Log::warning("Sanitized: $usernameSanitized");
            $user['username']=$usernameSanitized;
            $username=$usernameSanitized;
        }
        if(!isset($user['fullName'])){
            Log::info('User fullName not set, setting to username: '.$username);
            $user['fullName']=$username;
        }
        $user=UserManagement::createUser($user);
        Log::debug('Returning from RigakuImport::createUser');
        return $user;
    }

    /**
     * @throws BadRequestException
     * @throws ForbiddenException
     * @throws NotFoundException
     * @throws ServerException
     */
    public static function createImager($params){
        Log::debug('In RigakuImport::createImager');
        static::require(['name','nominalTemperature'], $params);
        $params['manufacturerName']='Rigaku';
        $imager=ImagingManagement::createImager($params);
        Log::debug('Returning from FormulatrixImport::createImager');
        return $imager;
    }


}