<?php

use PHPUnit\Framework\TestCase;

class SynchWebSynchrotronTest extends TestCase {

    public static $synchWebSynchrotrons;

    public static $authTokens=array();

    const HANDLER='SynchWeb';

    public function setUp():void {
        session::init(new DummySession());
        session::becomeAdmin();
        $user=user::getFirstAdmin();
        session::set('user', $user);
        session::set('userId', $user['id']);
        session::revertAdmin();
    }

    public function getSynchrotrons(){
        if(!isset(static::$synchWebSynchrotrons)){
            $config=@file_get_contents(__DIR__.'/../../../../testconfig.json');
            $config=json_decode($config,true);
            foreach ($config['synchrotrons'] as &$synchrotron){
                if($synchrotron['handler']===self::HANDLER){
                    static::$synchWebSynchrotrons[]=array($synchrotron);
                }
            }
        }
        return static::$synchWebSynchrotrons;
    }

    /**
     * @dataProvider getSynchrotrons
     * @param $synchrotron
     */
    public function testHandlerIsCorrect($synchrotron){
        self::assertEquals(self::HANDLER,$synchrotron['handler'], $synchrotron['name'].' is not an EXI synchrotron');
    }

    /**
     * @dataProvider getSynchrotrons
     * @param $synchrotron
     */
    public function testAuthenticate($synchrotron){
        $result=corsproxy::doPost(
            $synchrotron['baseuri'].'authenticate',
            array(
                'rawPostBody'=>'{ "login":"'.$synchrotron['user'].'", "password":"'.$synchrotron['pass'].'" }'
            ),
            false);
        self::assertNotEmpty($result, $synchrotron['name'].' returned an empty result');
        $result=json_decode($result, true);
        self::assertNotEmpty($result, $synchrotron['name'].' returned a non-JSON response');
        self::assertArrayHasKey('jwt', $result, $synchrotron['name'].' response does not have a JWT token');
        static::$authTokens[$synchrotron['name']]=$result['jwt'];
    }

    /**
     * @dataProvider getSynchrotrons
     * @param $synchrotron
     */
    public static function testGetProposals($synchrotron){
        self::assertArrayHasKey($synchrotron['name'], static::$authTokens, 'Auth token not saved for '.$synchrotron['name']);
        $token=static::$authTokens[$synchrotron['name']];
        $result=corsproxy::doGet(
            $synchrotron['baseuri'].'proposal?page=1&per_page=50&all=1&next=1',
            array('httpHeaders'=>['Authorization: Bearer '.$token]),
            false
        );
        self::assertNotEmpty($result, $synchrotron['name'].' returned an empty result');
        $result=json_decode($result, true);
        self::assertNotEmpty($result, $synchrotron['name'].' returned a non-JSON response');
        self::assertArrayNotHasKey('error', $result, $synchrotron['name'].' returned an error');
        self::assertArrayHasKey('total', $result, $synchrotron['name'].' returned no "total" key');
        self::assertArrayHasKey('total', $result, $synchrotron['name'].' returned no "data" key');
    }


}